#!perl.exe -w

############################################################################
# SendSerial
# Copyright (c) 2006 Factorycode <info@factorycode.com>
 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation;
 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS.
# IN NO EVENT SHALL THE COPYRIGHT HOLDER(S) AND AUTHOR(S) BE LIABLE FOR ANY
# CLAIM, OR ANY SPECIAL INDIRECT OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES 
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN 
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF 
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
# ALL LIABILITY, INCLUDING LIABILITY FOR INFRINGEMENT OF ANY PATENTS, 
# COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS, RELATING TO USE OF THIS 
# SOFTWARE IS DISCLAIMED.
############################################################################


#Information for the perl2exe program to enter in the properties field of the executable.
#perl2exe_info CompanyName=www.factorycode.com
#perl2exe_info FileDescription=Send_Serial
#perl2exe_info ProductName=SendSerial
#perl2exe_info ProductVersion=Not implemented.
#perl2exe_info LegalCopyright=Factorycode 2006
#perl2exe_info FileVersion=0.0.0.0

use Properties;
use strict;
use warnings;
use Carp::Heavy;
use send_serial;

# Global variables
my $properties = Properties->new;
my $numArgs=0;
my $savedARGV;

# Define constants for the program
use constant {
  TRUE  => 1,
  FALSE => 0,
};
my $noexit = FALSE;


sub die_with_prompt {
  if ($noexit) {
    print "Error caught - $_[0]\n";
    print "Press ENTER-key to exit.\n";
    my $ans = <STDIN>;
  }
}

$SIG{__DIE__} = \&die_with_prompt;

###############################################################################
# MAIN EXECUTION BELOW (NO FUNCTIONS)                                         #
###############################################################################

# Check if the first argument is an existing file or directory
sub shiftArgs {
  $numArgs = $#ARGV + 1;
  
  #print("Program '$0' invoked with $numArgs arguments:\n");
  $savedARGV = $0 . " "; #Program name + path
  foreach my $argnum (0 .. $#ARGV) {
   #print("\"$ARGV[$argnum]\"\n");
   $savedARGV = $savedARGV . "\"$ARGV[$argnum]\" ";
  }
  #print $savedARGV;
  if ( $numArgs < 1 ) {
    return TRUE;
  }

  my $arg;
  while ( $arg = shift @ARGV ) {
    if ( $arg =~ /^-p/ || $arg =~ /^--port/ ) 
    { 
	  $arg = shift @ARGV;
      $properties->addProperty("SIO::device",$arg);
    }
    elsif ( $arg =~ /-h/ || $arg =~ /--help/ ) {
	  print "$0 is a program to send characters to a serial port. They must be echoed buy the receiver or they will be retransmitted.\n";
	  print "Syntax: $0 [-p <COMx>] [Commandchar]\n\n";
	  print "-p <COMx>			- Set serial port to send to. Example '-p COM3'\n";
	  print "-n 				- Do not exit until Enter-key is pressed. Useful for debugging to see error messages.\n";
	  print "-h					- This helptext \n";
	  print "[CommandChar(s)]	- The command characters to send out on COM-port. Will be resent until they are echoed.\n";
	  print "\n Copyright Factorycode (C) 2006\n";
	  if ($noexit) {
        print "Press ENTER-key to exit.\n";
        my $ans = <STDIN>;
      }
	  exit 0;
    }
	elsif ( $arg =~ /-n/ || $arg =~ /--noexit/ ) {
      $noexit = TRUE;
	}
	else {
      #print "String to send: $arg\n";
	  $properties->addProperty("StringToSend",$arg);  
    }
  }
  
  return TRUE;
}

my $exitValue = 0;

# Exception handler at top level only.
eval {
  $properties->addProperty('SIO::device',"COM1");
  $properties->addProperty('SIO::baudrate', "115200");
  $properties->addProperty('SIO::parity', "none");
  $properties->addProperty('SIO::databits', "8");
  $properties->addProperty('SIO::stopbits', "1");
  $properties->addProperty('SIO::handshake', "none");

  shiftArgs();
  
  send_serial->initSIO($properties);
  
  my $result = "";
  if ($properties->getProperty('StringToSend')) {
	$result = send_serial->sendAndCheckEcho($properties->getProperty('StringToSend','DIE'));
  }
  else {
	print "Nothing to send?!?";
  }
  send_serial->closeSIO();
  
  if ( $result ) {
    #print "\n\nReceived $result";
    $exitValue = 0;
  }
  else {
    print "\n\n    No result returned. Fail.";
	$exitValue = 1;
  }
};

if ($@) {
  print "\n*** ABORTED: \n" . $@;
  $exitValue = 2;
}

if ($noexit) {
  print "Press ENTER-key to exit.\n";
  my $ans = <STDIN>;
}

exit $exitValue;




